@@ -40,16 +40,22 @@ class account_move_line(osv.osv):
         else:
             fiscalyear_clause = '%s' % context['fiscalyear']
         state=context.get('state',False)
-        where_move_state=''
+        where_move_state = ''
+        where_move_lines_by_date = ''
+
+        if context.get('date_from', False) and context.get('date_to', False):
+            where_move_lines_by_date = " AND " +obj+".move_id in ( select id from account_move  where date >= '" +context['date_from']+"' AND date <= '"+context['date_to']+"')"
+            
         if state:
             if state.lower() not in ['all']:
                 where_move_state= " AND "+obj+".move_id in (select id from account_move where account_move.state = '"+state+"')"
-
+        
+                
         if context.get('periods', False):
             ids = ','.join([str(x) for x in context['periods']])
-            return obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) AND id in (%s)) %s" % (fiscalyear_clause, ids,where_move_state)
+            return obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) AND id in (%s)) %s %s" % (fiscalyear_clause, ids,where_move_state,where_move_lines_by_date)
         else:
-            return obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) %s)" % (fiscalyear_clause,where_move_state)
+            return obj+".state<>'draft' AND "+obj+".period_id in (SELECT id from account_period WHERE fiscalyear_id in (%s) %s %s)" % (fiscalyear_clause,where_move_state,where_move_lines_by_date)
 
     def default_get(self, cr, uid, fields, context={}):
         data = self._default_get(cr, uid, fields, context)
@@ -342,8 +348,8 @@ class account_move_line(osv.osv):
         'statement_id': fields.many2one('account.bank.statement', 'Statement', help="The bank statement used for bank reconciliation", select=1),
         'reconcile_id': fields.many2one('account.move.reconcile', 'Reconcile', readonly=True, ondelete='set null', select=2),
         'reconcile_partial_id': fields.many2one('account.move.reconcile', 'Partial Reconcile', readonly=True, ondelete='set null', select=2),
-        'amount_currency': fields.float('Amount Currency', help="The amount expressed in an optionnal other currency if it is a multi-currency entry."),
-        'currency_id': fields.many2one('res.currency', 'Currency', help="The optionnal other currency if it is a multi-currency entry."),
+        'amount_currency': fields.float('Amount Currency', help="The amount expressed in an optional other currency if it is a multi-currency entry."),
+        'currency_id': fields.many2one('res.currency', 'Currency', help="The optional other currency if it is a multi-currency entry."),
 
         'period_id': fields.many2one('account.period', 'Period', required=True, select=2),
         'journal_id': fields.many2one('account.journal', 'Journal', required=True, select=1),
@@ -360,8 +366,9 @@ class account_move_line(osv.osv):
         'centralisation': fields.selection([('normal','Normal'),('credit','Credit Centralisation'),('debit','Debit Centralisation')], 'Centralisation', size=6),
         'balance': fields.function(_balance, method=True, string='Balance'),
         'state': fields.selection([('draft','Draft'), ('valid','Valid')], 'Status', readonly=True),
-        'tax_code_id': fields.many2one('account.tax.code', 'Tax Account'),
-        'tax_amount': fields.float('Tax/Base Amount', digits=(16,2), select=True),
+        'tax_code_id': fields.many2one('account.tax.code', 'Tax Account', help="The Account can either be a base tax code or tax code account."),
+        'tax_amount': fields.float('Tax/Base Amount', digits=(16,2), select=True, help="If the Tax account is tax code account, this field will contain the taxed amount.If the tax account is base tax code,\
+                    this field will contain the basic amount(without tax)."),
         'invoice': fields.function(_invoice, method=True, string='Invoice',
             type='many2one', relation='account.invoice', fnct_search=_invoice_search),
         'account_tax_id':fields.many2one('account.tax', 'Tax'),
@@ -531,6 +538,7 @@ class account_move_line(osv.osv):
 
     def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context={}):
         id_set = ','.join(map(str, ids))
+        
         lines = self.browse(cr, uid, ids, context=context)
         unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
         credit = debit = 0.0
@@ -565,7 +573,7 @@ class account_move_line(osv.osv):
             raise osv.except_osv(_('Error'), _('Entry is already reconciled'))
         account = self.pool.get('account.account').browse(cr, uid, account_id, context=context)
         if not context.get('fy_closing', False) and not account.reconcile:
-            raise osv.except_osv(_('Error'), _('The account is not defined to be reconcile !'))
+            raise osv.except_osv(_('Error'), _('The account is not defined to be reconciled !'))
         if r[0][1] != None:
             raise osv.except_osv(_('Error'), _('Some entries are already reconciled !'))
 
@@ -707,7 +715,6 @@ class account_move_line(osv.osv):
         return result
 
     def unlink(self, cr, uid, ids, context={}, check=True):
-        print "CC"
         self._update_check(cr, uid, ids, context)
         result = False
         for line in self.browse(cr, uid, ids, context):
@@ -793,6 +800,7 @@ class account_move_line(osv.osv):
             context['period_id'] = m.period_id.id
 
         self._update_journal_check(cr, uid, context['journal_id'], context['period_id'], context)
+        company_currency = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.currency_id.id
 
         move_id = vals.get('move_id', False)
         journal = self.pool.get('account.journal').browse(cr, uid, context['journal_id'])
@@ -829,7 +837,7 @@ class account_move_line(osv.osv):
             if journal.type_control_ids:
                 type = account.user_type
                 for t in journal.type_control_ids:
-                    if type==t.code:
+                    if type.code == t.code:
                         ok = True
                         break
             if journal.account_control_ids and not ok:
@@ -837,7 +845,7 @@ class account_move_line(osv.osv):
                     if a.id==vals['account_id']:
                         ok = True
                         break
-            if (account.currency_id) and 'amount_currency' not in vals:
+            if (account.currency_id) and 'amount_currency' not in vals and account.currency_id.id <> company_currency:
                 vals['currency_id'] = account.currency_id.id
                 cur_obj = self.pool.get('res.currency')
                 ctx = {}
@@ -849,7 +857,6 @@ class account_move_line(osv.osv):
         if not ok:
             raise osv.except_osv(_('Bad account !'), _('You can not use this general account in this journal !'))
 
-#        result = super(osv.osv, self).create(cr, uid, vals, context)
         if 'analytic_account_id' in vals and vals['analytic_account_id']:
             if journal.analytic_journal_id:
                 vals['analytic_lines'] = [(0,0, {
@@ -935,11 +942,12 @@ class account_move_line(osv.osv):
                     self.create(cr, uid, data, context)
             del vals['account_tax_id']
 
-        if not is_new_move and 'date' in vals:
-            if context and ('__last_update' in context):
-                del context['__last_update']
-            self.pool.get('account.move').write(cr, uid, [move_id], {'date':vals['date']}, context=context)
-        if check:
+        # No needed, related to the job
+        #if not is_new_move and 'date' in vals:
+        #    if context and ('__last_update' in context):
+        #        del context['__last_update']
+        #    self.pool.get('account.move').write(cr, uid, [move_id], {'date':vals['date']}, context=context)
+        if check and not context.get('no_store_function'):
             tmp = self.pool.get('account.move').validate(cr, uid, [vals['move_id']], context)
             if journal.entry_posted and tmp:
                 self.pool.get('account.move').button_validate(cr,uid, [vals['move_id']],context)
