@@ -58,6 +58,7 @@ class crm_case_section(osv.osv):
         'name': fields.char('Case Section',size=64, required=True, translate=True),
         'code': fields.char('Section Code',size=8),
         'active': fields.boolean('Active'),
+        'allow_unlink': fields.boolean('Allow Delete', help="Allows to delete non draft cases"),
         'sequence': fields.integer('Sequence'),
         'user_id': fields.many2one('res.users', 'Responsible User'),
         'reply_to': fields.char('Reply-To', size=64, help="The email address put in the 'Reply-To' of all emails sent by Open ERP about cases in this section"),
@@ -66,6 +67,7 @@ class crm_case_section(osv.osv):
     }
     _defaults = {
         'active': lambda *a: 1,
+        'allow_unlink': lambda *a: 1,
     }
     _sql_constraints = [
         ('code_uniq', 'unique (code)', 'The code of the section must be unique !')
@@ -238,6 +240,7 @@ class crm_case_rule(osv.osv):
 
         'trg_priority_from': fields.selection([('','')] + AVAILABLE_PRIORITIES, 'Minimum Priority'),
         'trg_priority_to': fields.selection([('','')] + AVAILABLE_PRIORITIES, 'Maximim Priority'),
+        'trg_max_history': fields.integer('Maximum Communication History'),
 
         'act_method': fields.char('Call Object Method', size=64),
         'act_state': fields.selection([('','')]+AVAILABLE_STATES, 'Set state to', size=16),
@@ -270,8 +273,8 @@ class crm_case_rule(osv.osv):
 
     def _check(self, cr, uid, ids=False, context={}):
         '''
-        Function called by the sceduler to proccess cases for date actions
-        Only works on not done and canceled cases
+        Function called by the scheduler to process cases for date actions
+        Only works on not done and cancelled cases
         '''
         cr.execute('select * from crm_case \
                 where (date_action_last<%s or date_action_last is null) \
@@ -327,7 +330,7 @@ class crm_case(osv.osv):
 
     _columns = {
         'id': fields.integer('ID', readonly=True),
-        'name': fields.char('Description',size=64, required=True),
+        'name': fields.char('Description',size=64,required=True),
         'priority': fields.selection(AVAILABLE_PRIORITIES, 'Priority'),
         'active': fields.boolean('Active'),
         'description': fields.text('Your action'),
@@ -386,14 +389,15 @@ class crm_case(osv.osv):
         'partner_id': _get_default_partner,
         'partner_address_id': _get_default_partner_address,
         'email_from': _get_default_email,
-        'state': lambda *a: 'open',
+        'state': lambda *a: 'draft',
         'priority': lambda *a: AVAILABLE_PRIORITIES[2][0],
+        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
     }
     _order = 'priority, date_deadline desc, date desc,id desc'
 
     def unlink(self, cr, uid, ids, context={}):
         for case in self.browse(cr, uid, ids, context):
-            if case.state <> 'draft':
+            if (not case.section_id.allow_unlink) and (case.state <> 'draft'):
                 raise osv.except_osv(_('Warning !'),
                     _('You can not delete this case. You should better cancel it.'))
         return super(crm_case, self).unlink(cr, uid, ids, context)
@@ -415,6 +419,7 @@ class crm_case(osv.osv):
                     ok = ok and (not action.trg_categ_id or action.trg_categ_id.id==case.categ_id.id)
                     ok = ok and (not action.trg_user_id.id or action.trg_user_id.id==case.user_id.id)
                     ok = ok and (not action.trg_partner_id.id or action.trg_partner_id.id==case.partner_id.id)
+                    ok = ok and (not action.trg_max_history or action.trg_max_history<=(len(case.history_line)+1))
                     ok = ok and (
                         not action.trg_partner_categ_id.id or
                         (
@@ -436,9 +441,9 @@ class crm_case(osv.osv):
                         else:
                             base = mx.DateTime.strptime(case.create_date[:19], '%Y-%m-%d %H:%M:%S')
                     elif action.trg_date_type=='deadline' and case.date_deadline:
-                        base = mx.DateTime.strptime(case.date_deadline, '%Y-%m-%d')
+                        base = mx.DateTime.strptime(case.date_deadline, '%Y-%m-%d %H:%M:%S')
                     elif action.trg_date_type=='date' and case.date:
-                        base = mx.DateTime.strptime(case.date, '%Y-%m-%d')
+                        base = mx.DateTime.strptime(case.date, '%Y-%m-%d %H:%M:%S')
                     if base:
                         fnct = {
                             'minutes': lambda interval: mx.DateTime.RelativeDateTime(minutes=interval),
@@ -476,7 +481,9 @@ class crm_case(osv.osv):
                             write['priority'] = action.act_priority
                         if action.act_email_cc:
                             if '@' in (case.email_cc or ''):
-                                write['email_cc'] = case.email_cc+','+action.act_email_cc
+                                emails = case.email_cc.split(",")
+                                if  action.act_email_cc not in emails:# and '<'+str(action.act_email_cc)+">" not in emails:
+                                    write['email_cc'] = case.email_cc+','+action.act_email_cc
                             else:
                                 write['email_cc'] = action.act_email_cc
                         write['date_action_last'] = time.strftime('%Y-%m-%d %H:%M:%S')
@@ -495,11 +502,12 @@ class crm_case(osv.osv):
                         if action.act_mail_to_partner:
                             emails.append(case.email_from)
                         if action.act_mail_to_watchers:
-                            emails += (case.email_cc or '').split(',')
+                            emails += (action.act_email_cc or '').split(',')
                         if action.act_mail_to_email:
                             emails += (action.act_mail_to_email or '').split(',')
                         emails = filter(None, emails)
                         if len(emails) and action.act_mail_body:
+                            emails = list(set(emails))                            
                             self.email_send(cr, uid, case, emails, action.act_mail_body)
                         break
             action_ids = newactions
@@ -507,13 +515,14 @@ class crm_case(osv.osv):
         return True
 
     def format_body(self, body):
-        return tools.ustr(body.encode('ascii', 'replace'))
+        return (body or u'').encode('utf8', 'replace')
 
     def format_mail(self, case, body):
         data = {
             'case_id': case.id,
             'case_subject': case.name,
             'case_date': case.date,
+            'case_description': case.description,
 
             'case_user': (case.user_id and case.user_id.name) or '/',
             'case_user_email': (case.user_id and case.user_id.address_id and case.user_id.address_id.email) or '/',
@@ -523,7 +532,7 @@ class crm_case(osv.osv):
             'partner': (case.partner_id and case.partner_id.name) or '/',
             'partner_email': (case.partner_address_id and case.partner_address_id.email) or '/',
         }
-        return self.format_body(body) % data
+        return self.format_body(body % data)
 
     def email_send(self, cr, uid, case, emails, body, context={}):
         body = self.format_mail(case, body)
@@ -531,7 +540,13 @@ class crm_case(osv.osv):
             emailfrom = case.user_id.address_id.email
         else:
             emailfrom = case.section_id.reply_to
-        tools.email_send(emailfrom, emails, '['+str(case.id)+'] '+case.name, body, reply_to=case.section_id.reply_to, tinycrm=str(case.id))
+        name = '[%d] %s' % (case.id, case.name.encode('utf8'))
+        reply_to = case.section_id.reply_to or False
+        if reply_to: reply_to = reply_to.encode('utf8')
+        if not emailfrom:
+            raise osv.except_osv(_('Error!'),
+                    _("No E-Mail ID Found for your Company address or missing reply address in section!"))
+        tools.email_send(emailfrom, emails, name, body, reply_to=reply_to, tinycrm=str(case.id))
         return True
     def __log(self, cr, uid, cases, keyword, context={}):
         if not self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_'+keyword):
@@ -597,6 +612,11 @@ class crm_case(osv.osv):
         for case in self.browse(cr, uid, ids):
             if case.section_id.reply_to and case.email_from:
                 src = case.email_from
+                
+                if not src:
+                    raise osv.except_osv(_('Error!'),
+                        _("No E-Mail ID Found for the Responsible Partner or missing reply address in section!"))
+                    
                 dest = case.section_id.reply_to
                 body = case.email_last or case.description
                 if not destination:
@@ -611,7 +631,7 @@ class crm_case(osv.osv):
                     attach_ids = self.pool.get('ir.attachment').search(cr, uid, [('res_model', '=', 'crm.case'), ('res_id', '=', case.id)])
                     attach_to_send = self.pool.get('ir.attachment').read(cr, uid, attach_ids, ['datas_fname','datas'])
                     attach_to_send = map(lambda x: (x['datas_fname'], base64.decodestring(x['datas'])), attach_to_send)
-
+                
                 # Send an email
                 tools.email_send(
                     src,
@@ -646,6 +666,9 @@ class crm_case(osv.osv):
             if not case.email_from:
                 raise osv.except_osv(_('Error!'),
                         _('You must put a Partner eMail to use this action!'))
+            if not case.user_id:
+                raise osv.except_osv(_('Error!'),
+                        _('You must define a responsible user for this case in order to use this action!'))
             if not case.description:
                 raise osv.except_osv(_('Error!'),
                         _('Can not send mail with empty body,you should have description in the body'))
@@ -661,8 +684,14 @@ class crm_case(osv.osv):
             body = case.description or ''
             if case.user_id.signature:
                 body += '\n\n%s' % (case.user_id.signature)
+            
+            emailfrom = case.user_id.address_id and case.user_id.address_id.email or False
+            if not emailfrom:
+                raise osv.except_osv(_('Error!'),
+                        _("No E-Mail ID Found for your Company address!"))
+                
             tools.email_send(
-                case.user_id.address_id.email,
+                emailfrom,
                 emails,
                 '['+str(case.id)+'] '+case.name,
                 self.format_body(body),
@@ -718,6 +747,7 @@ class crm_case(osv.osv):
             else:
                 raise osv.except_osv(_('Error !'), _('You can not escalate this case.\nYou are already at the top level.'))
             self.write(cr, uid, ids, data)
+        cases = self.browse(cr, uid, ids)
         self.__history(cr, uid, cases, _('Escalate'))
         self._action(cr, uid, cases, 'escalate')
         return True
@@ -790,6 +820,14 @@ class crm_case_history(osv.osv):
     _description = "Case history"
     _order = "id desc"
     _inherits = {'crm.case.log':"log_id"}
+
+    def create(self, cr, user, vals, context=None):
+        if vals.has_key('case_id') and vals['case_id']:
+            case_obj = self.pool.get('crm.case')
+            cases = case_obj.browse(cr, user, [vals['case_id']])
+            case_obj._action(cr, user, cases, '')
+        return super(crm_case_history, self).create(cr, user, vals, context)
+
     def _note_get(self, cursor, user, ids, name, arg, context=None):
         res = {}
         for hist in self.browse(cursor, user, ids, context or {}):
