@@ -38,10 +38,11 @@ import string
 from psycopg2 import Binary
 from tools import config
 import tools
+from tools.translate import _
 
 def random_name():
     random.seed()
-    d = [random.choice(string.letters) for x in xrange(10) ]
+    d = [random.choice(string.ascii_letters) for x in xrange(10) ]
     name = "".join(d)
     return name
 
@@ -65,8 +66,10 @@ def random_name():
 #   root: if we are at the first directory of a ressource
 #
 INVALID_CHARS={'*':str(hash('*')), '|':str(hash('|')) , "\\":str(hash("\\")), '/':'__', ':':str(hash(':')), '"':str(hash('"')), '<':str(hash('<')) , '>':str(hash('>')) , '?':str(hash('?'))}
+
+
 class node_class(object):
-    def __init__(self, cr, uid, path,object,object2=False, context={}, content=False, type='collection', root=False):
+    def __init__(self, cr, uid, path, object, object2=False, context={}, content=False, type='collection', root=False):
         self.cr = cr
         self.uid = uid
         self.path = path
@@ -91,35 +94,46 @@ class node_class(object):
             where.append( ('parent_id','=',self.object.id) )
             where.append( ('res_id','=',False) )
         if nodename:
-            where.append( (fobj._rec_name,'like',nodename) )
+            where.append( (fobj._rec_name,'=',nodename) )
         for content in self.object.content_ids:
             if self.object2 or not content.include_name:
                 if content.include_name:
-                    test_nodename = self.object2.name + (content.suffix or '') + (content.extension or '')
+                    content_name = self.object2.name
+                    obj = pool.get(self.object.ressource_type_id.model)
+                    name_for = obj._name.split('.')[-1]            
+                    if content_name  and content_name.find(name_for) == 0  :
+                        content_name = content_name.replace(name_for,'')
+                    test_nodename = content_name + (content.suffix or '') + (content.extension or '')
                 else:
                     test_nodename = (content.suffix or '') + (content.extension or '')
                 if test_nodename.find('/'):
                     test_nodename=test_nodename.replace('/', '_')
                 path = self.path+'/'+test_nodename
                 if not nodename:
-                    n = node_class(self.cr, self.uid,path, self.object2, False, content=content, type='content', root=False)
+                    n = node_class(self.cr, self.uid,path, self.object2, False, context=self.context, content=content, type='content', root=False)
                     res2.append( n)
                 else:
                     if nodename == test_nodename:
-                        n = node_class(self.cr, self.uid, path, self.object2, False, content=content, type='content', root=False)
+                        n = node_class(self.cr, self.uid, path, self.object2, False, context=self.context, content=content, type='content', root=False)
                         res2.append(n)
 
-        ids = fobj.search(self.cr, self.uid, where+[ ('parent_id','=',self.object and self.object.id or False) ], context=self.context)
+        ids = fobj.search(self.cr, self.uid, where+[ ('parent_id','=',self.object and self.object.id or False) ])
         if self.object and self.root and (self.object.type=='ressource'):
-            ids += fobj.search(self.cr, self.uid, where+[ ('parent_id','=',False) ], context=self.context)
+            ids += fobj.search(self.cr, self.uid, where+[ ('parent_id','=',False) ])
         res = fobj.browse(self.cr, self.uid, ids, context=self.context)
-        return map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name, x, False, type='file', root=False), res) + res2
-
+        return map(lambda x: node_class(self.cr, self.uid, self.path+'/'+eval('x.'+fobj._rec_name), x, False, context=self.context, type='file', root=False), res) + res2
+    
+    def get_translation(self,value,lang):
+        result = value
+        #TODO : to get translation term        
+        return result 
+    
     def directory_list_for_child(self,nodename,parent=False):
         pool = pooler.get_pool(self.cr.dbname)
         where = []
-        if nodename:
-            where.append(('name','like',nodename))
+        if nodename:    
+            nodename = self.get_translation(nodename, self.context['lang'])            
+            where.append(('name','=',nodename))
         if (self.object and self.object.type=='directory') or not self.object2:
             where.append(('parent_id','=',self.object and self.object.id or False))
         else:
@@ -129,43 +143,36 @@ class node_class(object):
         else:
             where.append(('ressource_parent_type_id','=',False))
 
-        ids = pool.get('document.directory').search(self.cr, self.uid, where+[('ressource_id','=',0)], self.context)
+        ids = pool.get('document.directory').search(self.cr, self.uid, where+[('ressource_id','=',0)])
         if self.object2:
-            ids += pool.get('document.directory').search(self.cr, self.uid, where+[('ressource_id','=',self.object2.id)], self.context)
-        res = pool.get('document.directory').browse(self.cr, self.uid, ids,self.context)
+            ids += pool.get('document.directory').search(self.cr, self.uid, where+[('ressource_id','=',self.object2.id)])        
+        res = pool.get('document.directory').browse(self.cr, self.uid, ids, self.context)
         return res
 
-    def _child_get(self, nodename=False):
+    def _child_get(self, nodename=False):        
         if self.type not in ('collection','database'):
             return []
         res = self.directory_list_for_child(nodename)
-        result= map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name, x, x.type=='directory' and self.object2 or False, root=self.root), res)
+        result= map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name, x, x.type=='directory' and self.object2 or False, context=self.context, root=self.root), res)
         if self.type=='database':
             pool = pooler.get_pool(self.cr.dbname)
             fobj = pool.get('ir.attachment')
             vargs = [('parent_id','=',False),('res_id','=',False)]
             if nodename:
-                vargs.append(('name','like',nodename))
+                vargs.append((fobj._rec_name,'=',nodename))
             file_ids=fobj.search(self.cr,self.uid,vargs)
 
             res = fobj.browse(self.cr, self.uid, file_ids, context=self.context)
-            result +=map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name, x, False, type='file', root=self.root), res)
+            result +=map(lambda x: node_class(self.cr, self.uid, self.path+'/'+eval('x.'+fobj._rec_name), x, False, context=self.context, type='file', root=self.root), res)
         if self.type=='collection' and self.object.type=="ressource":
-            where = self.object.domain and eval(self.object.domain, {'active_id':self.root}) or []
-            pool = pooler.get_pool(self.cr.dbname)
+            where = self.object.domain and eval(self.object.domain, {'active_id':self.root, 'uid':self.uid}) or []
+            pool = pooler.get_pool(self.cr.dbname)            
             obj = pool.get(self.object.ressource_type_id.model)
+            _dirname_field = obj._rec_name
+            if len(obj.fields_get(self.cr, self.uid, ['dirname'])):
+                _dirname_field = 'dirname'            
 
-            if self.object.ressource_tree:
-                if obj._parent_name in obj.fields_get(self.cr,self.uid):
-                    where.append((obj._parent_name,'=',self.object2 and self.object2.id or False))
-                else :
-                    if self.object2:
-                        return result
-            else:
-                if self.object2:
-                    return result
-
-            name_for = obj._name.split('.')[-1]
+            name_for = obj._name.split('.')[-1]            
             if nodename  and nodename.find(name_for) == 0  :
                 id = int(nodename.replace(name_for,''))
                 where.append(('id','=',id))
@@ -175,18 +182,42 @@ class node_class(object):
                 for invalid in INVALID_CHARS:
                     if nodename.find(INVALID_CHARS[invalid]) :
                         nodename=nodename.replace(INVALID_CHARS[invalid],invalid)
-                where.append(('name','like',nodename))
-            ids = obj.search(self.cr, self.uid, where, self.context)
+                nodename = self.get_translation(nodename, self.context['lang'])
+                where.append((_dirname_field,'=',nodename))
+
+            if self.object.ressource_tree:
+                if obj._parent_name in obj.fields_get(self.cr,self.uid):                    
+                    where.append((obj._parent_name,'=',self.object2 and self.object2.id or False))
+                    ids = obj.search(self.cr, self.uid, where)
+                    res = obj.browse(self.cr, self.uid, ids,self.context)
+                    result+= map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name.replace('/','__'), self.object, x, context=self.context, root=x.id), res)
+                    return result
+                else :
+                    if self.object2:
+                        return result
+            else:
+                if self.object2:
+                    return result
+
+            
+            ids = obj.search(self.cr, self.uid, where)
             res = obj.browse(self.cr, self.uid, ids,self.context)
-            for r in res:
+            for r in res:                               
+                if len(obj.fields_get(self.cr, self.uid, [_dirname_field])):
+                    r.name = eval('r.'+_dirname_field)
+                else:
+                    r.name = False
                 if not r.name:
-                    r.name = name_for+'%d'%r.id
+                    r.name = name_for + '%d'%r.id               
                 for invalid in INVALID_CHARS:
                     if r.name.find(invalid) :
-                        r.name=r.name.replace(invalid,INVALID_CHARS[invalid])
-            result2 = map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name.replace('/','__'), self.object, x, root=r.id), res)
+                        r.name = r.name.replace(invalid,INVALID_CHARS[invalid])            
+            result2 = map(lambda x: node_class(self.cr, self.uid, self.path+'/'+x.name.replace('/','__'), self.object, x, context=self.context, root=x.id), res)
             if result2:
-                result = result2
+                if self.object.ressource_tree:
+                    result += result2
+                else:
+                    result = result2                  
         return result
 
     def children(self):
@@ -262,7 +293,8 @@ class document_directory(osv.osv):
         model_ids=self.pool.get('ir.model').search(cr,uid,[('model','=',res_model)])
         if directory:
             _parent(dir_id,path)
-            path.append(self.pool.get(directory.ressource_type_id.model).browse(cr,uid,res_id).name)
+            if res_id:
+                path.append(self.pool.get(directory.ressource_type_id.model).browse(cr,uid,res_id).name)
             user=self.pool.get('res.users').browse(cr,uid,uid)
             return "ftp://%s:%s@localhost:%s/%s/%s"%(user.login,user.password,config.get('ftp_server_port',8021),cr.dbname,'/'.join(path))
         return False
@@ -283,7 +315,6 @@ class document_directory(osv.osv):
     def __init__(self, *args, **kwargs):
         res = super(document_directory, self).__init__(*args, **kwargs)
         self._cache = {}
-        return res
 
     def onchange_content_id(self, cr, uid, ids, ressource_type_id):
         return {}
@@ -291,7 +322,8 @@ class document_directory(osv.osv):
     def _get_childs(self, cr, uid, node, nodename=False, context={}):
         where = []
         if nodename:
-            where.append(('name','like',nodename))
+            nodename = self.get_translation(nodename, self.context['lang'])
+            where.append(('name','=',nodename))
         if object:
             where.append(('parent_id','=',object.id))
         ids = self.search(cr, uid, where, context)
@@ -305,12 +337,13 @@ class document_directory(osv.osv):
             object: the object.directory or object.directory.content
             object2: the other object linked (if object.directory.content)
     """
-    def get_object(self, cr, uid, uri, context={}):
+    def get_object(self, cr, uid, uri, context={}):        
+        #TODO : set user's context_lang in context        
         if not uri:
-            return node_class(cr, uid, '', False, type='database')
+            return node_class(cr, uid, '', False, context=context, type='database')
         turi = tuple(uri)
         if False and (turi in self._cache):
-            (path, oo, oo2, content,type,root) = self._cache[turi]
+            (path, oo, oo2, context, content,type,root) = self._cache[turi]
             if oo:
                 object = self.pool.get(oo[0]).browse(cr, uid, oo[1], context)
             else:
@@ -319,10 +352,10 @@ class document_directory(osv.osv):
                 object2 = self.pool.get(oo2[0]).browse(cr, uid, oo2[1], context)
             else:
                 object2 = False
-            node = node_class(cr, uid, path, object,object2, context, content, type, root)
+            node = node_class(cr, uid, '/', False, context=context, type='database')
             return node
 
-        node = node_class(cr, uid, '/', False, type='database')
+        node = node_class(cr, uid, '/', False, context=context, type='database')
         for path in uri[:]:
             if path:
                 node = node.child(path)
@@ -330,7 +363,7 @@ class document_directory(osv.osv):
                     return False
         oo = node.object and (node.object._name, node.object.id) or False
         oo2 = node.object2 and (node.object2._name, node.object2.id) or False
-        self._cache[turi] = (node.path, oo, oo2, node.content,node.type,node.root)
+        self._cache[turi] = (node.path, oo, oo2, node.context, node.content,node.type,node.root)
         return node
 
     def get_childs(self, cr, uid, uri, context={}):
@@ -357,7 +390,7 @@ class document_directory(osv.osv):
         ressource_parent_type_id=vals.get('ressource_parent_type_id',False)
         ressource_id=vals.get('ressource_id',0)
         if op=='write':
-           for directory in self.browse(cr,uid,ids):
+            for directory in self.browse(cr,uid,ids):
                 if not name:
                     name=directory.name
                 if not parent_id:
@@ -376,14 +409,14 @@ class document_directory(osv.osv):
         return True
     def write(self, cr, uid, ids, vals, context=None):
         if not self._check_duplication(cr,uid,vals,ids,op='write'):
-            raise osv.except_osv('ValidateError', _('Directory name must be unique!'))
+            raise osv.except_osv(_('ValidateError'), _('Directory name must be unique!'))
         return super(document_directory,self).write(cr,uid,ids,vals,context=context)
 
     def create(self, cr, uid, vals, context=None):
         if not self._check_duplication(cr,uid,vals):
-            raise osv.except_osv('ValidateError', _('Directory name must be unique!'))
+            raise osv.except_osv(_('ValidateError'), _('Directory name must be unique!'))
         if vals.get('name',False) and (vals.get('name').find('/')+1 or vals.get('name').find('@')+1 or vals.get('name').find('$')+1 or vals.get('name').find('#')+1) :
-            raise osv.except_osv('ValidateError', _('Directory name contains special characters!'))
+            raise osv.except_osv(_('ValidateError'), _('Directory name contains special characters!'))
         return super(document_directory,self).create(cr, uid, vals, context)
 
 document_directory()
@@ -484,7 +517,7 @@ def create_directory(path):
 
 class document_file(osv.osv):
     _inherit = 'ir.attachment'
-
+    _rec_name = 'datas_fname'
     def _get_filestore(self, cr):
         return os.path.join(tools.config['root_path'], 'filestore', cr.dbname)
 
@@ -492,11 +525,11 @@ class document_file(osv.osv):
         result = {}
         cr.execute('select id,store_fname,link from ir_attachment where id in ('+','.join(map(str,ids))+')')
         for id,r,l in cr.fetchall():
-	    try:
-	        value = file(os.path.join(self._get_filestore(cr), r), 'rb').read()
-	        result[id] = base64.encodestring(value)
-	    except:
-	        result[id]=''
+            try:
+                value = file(os.path.join(self._get_filestore(cr), r), 'rb').read()
+                result[id] = base64.encodestring(value)
+            except:
+                result[id]=''
 
             if context.get('bin_size', False):
                 result[id] = tools.human_size(len(result[id]))
@@ -516,7 +549,7 @@ class document_file(osv.osv):
                 try:
                     os.makedirs(path)
                 except:
-                    raise except_orm('Permission Denied !', 'You do not permissions to write on the server side.')
+                    raise except_orm(_('Permission Denied !'), _('You do not permissions to write on the server side.'))
 
             flag = None
             # This can be improved
@@ -534,7 +567,7 @@ class document_file(osv.osv):
             cr.execute('update ir_attachment set store_fname=%s,store_method=%s,file_size=%s where id=%s', (os.path.join(flag,filename),'fs',len(v),id))
             return True
         except Exception,e :
-            raise except_orm('Error!', str(e))
+            raise except_orm(_('Error!'), str(e))
 
     _columns = {
         'user_id': fields.many2one('res.users', 'Owner', select=1),
@@ -570,7 +603,7 @@ class document_file(osv.osv):
         res_model=vals.get('res_model',False)
         res_id=vals.get('res_id',0)
         if op=='write':
-           for file in self.browse(cr,uid,ids):
+            for file in self.browse(cr,uid,ids):
                 if not name:
                     name=file.name
                 if not parent_id:
@@ -581,7 +614,7 @@ class document_file(osv.osv):
                     res_id=file.res_id and file.res_id or 0
                 res=self.search(cr,uid,[('id','<>',file.id),('name','=',name),('parent_id','=',parent_id),('res_model','=',res_model),('res_id','=',res_id)])
                 if len(res):
-	                return False
+                    return False
         if op=='create':
             res=self.search(cr,uid,[('name','=',name),('parent_id','=',parent_id),('res_id','=',res_id),('res_model','=',res_model)])
             if len(res):
@@ -598,7 +631,7 @@ class document_file(osv.osv):
         if not len(res):
             return False
         if not self._check_duplication(cr,uid,vals,ids,'write'):
-            raise except_orm('ValidateError', 'File name must be unique!')
+            raise except_orm(_('ValidateError'), _('File name must be unique!'))
         result = super(document_file,self).write(cr,uid,ids,vals,context=context)
         cr.commit()
         try:
@@ -626,7 +659,8 @@ class document_file(osv.osv):
             result = obj_model.read(cr, uid, [vals['res_id']], context=context)
             if len(result):
                 obj=result[0]
-                vals['title'] = (obj['name'] or '')[:60]
+                if obj.get('name',False):
+                    vals['title'] = (obj.get('name',''))[:60]
                 if obj_model._name=='res.partner':
                     vals['partner_id']=obj['id']
                 elif obj.get('address_id',False):
@@ -648,10 +682,11 @@ class document_file(osv.osv):
             import urllib
             datas=base64.encodestring(urllib.urlopen(vals['link']).read())
         else:
-            datas=vals.get('datas',False)
-        vals['file_size']= len(datas)
+            datas = vals.get('datas',False)
+        
+        vals['file_size']= datas and len(datas) or 0
         if not self._check_duplication(cr,uid,vals):
-            raise except_orm('ValidateError', 'File name must be unique!')
+            raise except_orm(_('ValidateError'), _('File name must be unique!'))
         result = super(document_file,self).create(cr, uid, vals, context)
         cr.commit()
         try:
@@ -681,9 +716,45 @@ class document_configuration_wizard(osv.osv_memory):
         'host': fields.char('Server Address', size=64, help="Put here the server address or IP. " \
             "Keep localhost if you don't know what to write.", required=True)
     }
+
+    def detect_ip_addr(self, cr, uid, context=None):
+        def _detect_ip_addr(self, cr, uid, context=None):
+            from array import array
+            import socket
+            from struct import pack, unpack
+
+            try:
+                import fcntl
+            except ImportError:
+                fcntl = None
+
+            if not fcntl: # not UNIX:
+                host = socket.gethostname()
+                ip_addr = socket.gethostbyname(host)
+            else: # UNIX:
+                # get all interfaces:
+                nbytes = 128 * 32
+                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+                names = array('B', '\0' * nbytes)
+                outbytes = unpack('iL', fcntl.ioctl( s.fileno(), 0x8912, pack('iL', nbytes, names.buffer_info()[0])))[0]
+                namestr = names.tostring()
+                ifaces = [namestr[i:i+32].split('\0', 1)[0] for i in range(0, outbytes, 32)]
+
+                for ifname in [iface for iface in ifaces if iface != 'lo']:
+                    ip_addr = socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915, pack('256s', ifname[:15]))[20:24])
+                    break
+            return ip_addr
+
+        try:
+            ip_addr = _detect_ip_addr(self, cr, uid, context)
+        except:
+            ip_addr = 'localhost'
+        return ip_addr
+
     _defaults = {
-        'host': lambda *args: 'localhost'
+        'host': detect_ip_addr,
     }
+
     def action_cancel(self,cr,uid,ids,conect=None):
         return {
                 'view_type': 'form',
@@ -768,6 +839,6 @@ class document_configuration_wizard(osv.osv_memory):
                 "view_mode": 'form',
                 'res_model': 'ir.actions.configuration.wizard',
                 'type': 'ir.actions.act_window',
-                'target':'new',
+                'target': 'new',
         }
 document_configuration_wizard()
