@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 import os
 import time
 from tarfile import filemode
@@ -11,10 +12,53 @@ import pooler
 import netsvc
 import os
 from service import security
+from osv import osv
+import stat
 
 def log(message):
     logger = netsvc.Logger()
     logger.notifyChannel('DMS', netsvc.LOG_ERROR, message)
+
+
+def _get_month_name(month):
+    month=int(month)
+    if month==1:return 'Jan'
+    elif month==2:return 'Feb'
+    elif month==3:return 'Mar'
+    elif month==4:return 'Apr'
+    elif month==5:return 'May'
+    elif month==6:return 'Jun'
+    elif month==7:return 'Jul'
+    elif month==8:return 'Aug'
+    elif month==9:return 'Sep'
+    elif month==10:return 'Oct'
+    elif month==11:return 'Nov'
+    elif month==12:return 'Dec'
+
+def _to_unicode(s):
+    try:
+        return s.decode('utf-8')
+    except UnicodeError:
+        try:
+            return s.decode('latin')
+        except UnicodeError:
+            try:
+                return s.encode('ascii')
+            except UnicodeError:
+                return s
+
+def _to_decode(s):
+    try:
+        return s.encode('utf-8')
+    except UnicodeError:
+        try:
+            return s.encode('latin')
+        except UnicodeError:
+            try:
+                return s.decode('ascii')
+            except UnicodeError:
+                return s  
+    
 			
 class file_wrapper(StringIO.StringIO):
     def __init__(self, sstr='', ressource_id=False, dbname=None, uid=1, name=''):
@@ -83,16 +127,20 @@ class abstracted_fs:
         for db_name in result:
             db, cr = None, None
             try:
-                db = pooler.get_db_only(db_name)
-                cr = db.cursor()
-                cr.execute("SELECT 1 FROM pg_class WHERE relkind = 'r' AND relname = 'ir_module_module'")
-                if not cr.fetchone():
-                    continue
-
-                cr.execute("select id from ir_module_module where name like 'document%' and state='installed' ")
-                res = cr.fetchone()
-                if res and len(res):
-                    self.db_name_list.append(db_name)
+                try:
+                    db = pooler.get_db_only(db_name)
+                    cr = db.cursor()
+                    cr.execute("SELECT 1 FROM pg_class WHERE relkind = 'r' AND relname = 'ir_module_module'")
+                    if not cr.fetchone():
+                        continue
+    
+                    cr.execute("select id from ir_module_module where name like 'document%' and state='installed' ")
+                    res = cr.fetchone()
+                    if res and len(res):
+                        self.db_name_list.append(db_name)
+                    cr.commit()
+                except Exception, e:
+                    log(e)
             finally:
                 if cr is not None:
                     cr.close()
@@ -141,18 +189,28 @@ class abstracted_fs:
     # Ok
     def ftp2fs(self, path_orig, data):
         path = self.ftpnorm(path_orig)
-        if path and path=='/':
-            return None
+        if not data or (path and path=='/'):
+            return None               
         path2 = filter(None,path.split('/'))[1:]
         (cr, uid, pool) = data
+        if len(path2):     
+            path2[-1]=_to_unicode(path2[-1])
         res = pool.get('document.directory').get_object(cr, uid, path2[:])
         if not res:
             raise OSError(2, 'Not such file or directory.')
         return res
 
     # Ok
-    def fs2ftp(self, node):
-        res = node and ('/' + node.cr.dbname + '/' + node.path.encode('ascii','replace').replace('?','_')) or '/'
+    def fs2ftp(self, node):        
+        res='/'
+        if node:
+            res=os.path.normpath(node.path)
+            res = res.replace("\\", "/")        
+            while res[:2] == '//':
+                res = res[1:]
+            res='/' + node.cr.dbname + '/' + _to_decode(res)            
+            
+        #res = node and ('/' + node.cr.dbname + '/' + _to_decode(self.ftpnorm(node.path))) or '/'
         return res
 
     # Ok
@@ -172,6 +230,7 @@ class abstracted_fs:
 
     # Ok
     def create(self, node, objname, mode):
+        objname=_to_unicode(objname)
         cr = node.cr
         uid = node.uid
         pool = pooler.get_pool(cr.dbname)
@@ -193,7 +252,7 @@ class abstracted_fs:
             cid=False
 
             where=[('name','=',objname)]
-            if object and (object.type in ('directory','ressource')) or object2:
+            if object and (object.type in ('directory')) or object2:
                 where.append(('parent_id','=',object.id))
             else:
                 where.append(('parent_id','=',False))
@@ -212,7 +271,7 @@ class abstracted_fs:
                     'file_size': 0L,
                     'file_type': ext,
                 }
-                if object and (object.type in ('directory','ressource')) or not object2:
+                if object and (object.type in ('directory')) or not object2:
                     val['parent_id']= object and object.id or False
                 partner = False
                 if object2:
@@ -230,7 +289,7 @@ class abstracted_fs:
 
             s = file_wrapper('', cid, cr.dbname, uid, )
             return s
-        except Exception,e:
+        except Exception,e:             
             log(e)
             raise OSError(1, 'Operation not permited.')
 
@@ -293,7 +352,7 @@ class abstracted_fs:
 
 
     # Ok
-    def chdir(self, path):
+    def chdir(self, path):        
         if not path:
             self.cwd='/'
             return None
@@ -308,6 +367,7 @@ class abstracted_fs:
         if not node:
             raise OSError(1, 'Operation not permited.')
         try:
+            basename=_to_unicode(basename)
             object2=node and node.object2 or False
             object=node and node.object or False
             cr = node.cr
@@ -320,7 +380,7 @@ class abstracted_fs:
                 'ressource_parent_type_id': object and object.ressource_type_id.id or False,
                 'ressource_id': object2 and object2.id or False
             }
-            if (object and (object.type in ('directory'))) or not object2:
+            if (object and (object.type in ('directory'))) or not object2:                
                 val['parent_id'] =  object and object.id or False
             # Check if it alreayd exists !
             pool.get('document.directory').create(cr, uid, val)
@@ -341,6 +401,8 @@ class abstracted_fs:
         if path=='/':
             return None
         dbname = path.split('/')[1]
+        if dbname not in self.db_list():
+            return None
         try:
             db,pool = pooler.get_db_and_pool(dbname)
         except:
@@ -363,7 +425,12 @@ class abstracted_fs:
         if path is None:
             result = []
             for db in self.db_list():
-                result.append(false_node(db))
+                try:
+                    uid = security.login(db, self.username, self.password)
+                    if uid:
+                        result.append(false_node(db))                    
+                except osv.except_osv:                    
+                    pass
             return result
         return path.children()
 
@@ -408,6 +475,7 @@ class abstracted_fs:
             * A directory: change the parent and reassign childs to ressource
         """
         try:
+            dst_basename=_to_unicode(dst_basename)
             if src.type=='collection':
                 if src.object._table_name <> 'document.directory':
                     raise OSError(1, 'Operation not permited.')
@@ -447,11 +515,12 @@ class abstracted_fs:
                     ressource_type_id = pool.get('ir.model').search(cr,uid,[('model','=',dst_basedir.object2._name)])[0]
                     ressource_id = dst_basedir.object2.id
                     title = dst_basedir.object2.name
-                    ressource_model = dst_basedir.object2._name
+                    ressource_model = dst_basedir.object2._name                    
                     if dst_basedir.object2._name=='res.partner':
                         partner_id=dst_basedir.object2.id
                     else:
-                        partner_id= dst_basedir.object2.partner_id and dst_basedir.object2.partner_id.id or False
+                        obj2=pool.get(dst_basedir.object2._name)                         
+                        partner_id= obj2.fields_get(cr,uid,['partner_id']) and dst_basedir.object2.partner_id.id or False
                 else:
                     ressource_type_id = False
                     ressource_id=False
@@ -461,7 +530,7 @@ class abstracted_fs:
 
                 pool.get('document.directory').write(cr, uid, result['directory'], {
                     'ressource_id': ressource_id,
-                    'ressource_parent_type_id': ressource_type_id
+                    'ressource_type_id': ressource_type_id
                 })
                 val = {
                     'res_id': ressource_id,
@@ -497,7 +566,8 @@ class abstracted_fs:
                     if dst_basedir.object2._name=='res.partner':
                         val['partner_id']=dst_basedir.object2.id
                     else:
-                        val['partner_id']= dst_basedir.object2.partner_id and dst_basedir.object2.partner_id.id or False
+                        obj2=pool.get(dst_basedir.object2._name) 
+                        val['partner_id']= obj2.fields_get(src.cr, src.uid,['partner_id']) and dst_basedir.object2.partner_id.id or False
                 elif src.object.res_id:
                     # I had to do that because writing False to an integer writes 0 instead of NULL
                     # change if one day we decide to improve osv/fields.py
@@ -610,7 +680,7 @@ class abstracted_fs:
     def get_list_dir(self, path):
         """"Return an iterator object that yields a directory listing
         in a form suitable for LIST command.
-        """
+        """        
         if self.isdir(path):
             listing = self.listdir(path)
             #listing.sort()
@@ -645,7 +715,7 @@ class abstracted_fs:
                     listing.sort()
                 return self.format_list(basedir, listing)
 
-    # Ok
+    # Ok    
     def format_list(self, basedir, listing, ignore_err=True):
         """Return an iterator object that yields the entries of given
         directory emulating the "/bin/ls -lA" UNIX command output.
@@ -684,13 +754,15 @@ class abstracted_fs:
             # stat.st_mtime could fail (-1) if last mtime is too old
             # in which case we return the local time as last mtime
             try:
-                mtime = time.strftime("%b %d %H:%M", time.localtime(st.st_mtime))
+                mname=_get_month_name(time.strftime("%m", time.localtime(st.st_mtime)))               
+                mtime = mname+' '+time.strftime("%d %H:%M", time.localtime(st.st_mtime))
             except ValueError:
-                mtime = time.strftime("%b %d %H:%M")
-
-            # formatting is matched with proftpd ls output
+                mname=_get_month_name(time.strftime("%m"))
+                mtime = mname+' '+time.strftime("%d %H:%M")            
+            # formatting is matched with proftpd ls output            
+            path=_to_decode(file.path) #file.path.encode('ascii','replace').replace('?','_')                    
             yield "%s %3s %-8s %-8s %8s %s %s\r\n" %(perms, nlinks, uname, gname,
-                                                     size, mtime, file.path.encode('ascii','replace').replace('?','_').split('/')[-1])
+                                                     size, mtime, path.split('/')[-1])
 
     # Ok
     def format_mlsx(self, basedir, listing, perms, facts, ignore_err=True):
@@ -726,8 +798,7 @@ class abstracted_fs:
         if 'd' in perms:
             permdir += 'p'
         type = size = perm = modify = create = unique = mode = uid = gid = ""
-        for basename in listing:
-            file = os.path.join(basedir, basename)
+        for file in listing:                        
             try:
                 st = self.stat(file)
             except OSError:
@@ -737,12 +808,7 @@ class abstracted_fs:
             # type + perm
             if stat.S_ISDIR(st.st_mode):
                 if 'type' in facts:
-                    if basename == '.':
-                        type = 'type=cdir;'
-                    elif basename == '..':
-                        type = 'type=pdir;'
-                    else:
-                        type = 'type=dir;'
+                    type = 'type=dir;'                    
                 if 'perm' in facts:
                     perm = 'perm=%s;' %permdir
             else:
@@ -784,7 +850,9 @@ class abstracted_fs:
             # on Windows NTFS filesystems MTF records could be used).
             if 'unique' in facts:
                 unique = "unique=%x%x;" %(st.st_dev, st.st_ino)
-
+            path=_to_decode(file.path)
             yield "%s%s%s%s%s%s%s%s%s %s\r\n" %(type, size, perm, modify, create,
-                                                mode, uid, gid, unique, basename)
+                                                mode, uid, gid, unique, path)
+
+# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
 
