@@ -53,6 +53,7 @@ def _symbol_set(symb):
 class _column(object):
     _classic_read = True
     _classic_write = True
+    _prefetch = True
     _properties = False
     _type = 'unknown'
     _obj = None
@@ -196,6 +197,7 @@ class binary(_column):
     _symbol_get = lambda self, x: x and str(x)
 
     _classic_read = False
+    _prefetch = False
 
     def __init__(self, string='unknown', filters=None, **args):
         _column.__init__(self, string=string, **args)
@@ -308,11 +310,9 @@ class many2one(_column):
             names = dict(obj.name_get(cr, user, filter(None, res.values()), context))
         except except_orm:
             names = {}
-
             iids = filter(None, res.values())
-            cr.execute('select id,'+obj._rec_name+' from '+obj._table+' where id in ('+','.join(map(str, iids))+')')
-            for res22 in cr.fetchall():
-                names[res22[0]] = res22[1]
+            for iiid in iids:
+                names[iiid] = '// Access Denied //'
 
         for r in res.keys():
             if res[r] and res[r] in names:
@@ -352,6 +352,7 @@ class many2one(_column):
 class one2many(_column):
     _classic_read = False
     _classic_write = False
+    _prefetch = False
     _type = 'one2many'
 
     def __init__(self, obj, fields_id, string='unknown', limit=None, **args):
@@ -365,12 +366,15 @@ class one2many(_column):
     def get_memory(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
         if not context:
             context = {}
+        if self._context:
+            context = context.copy()
+            context.update(self._context)
         if not values:
             values = {}
         res = {}
         for id in ids:
             res[id] = []
-        ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id, 'in', ids)], limit=self._limit)
+        ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id, 'in', ids)], limit=self._limit, context=context)
         for r in obj.pool.get(self._obj).read(cr, user, ids2, [self._fields_id], context=context, load='_classic_write'):
             if r[self._fields_id] in res:
                 res[r[self._fields_id]].append(r['id'])
@@ -379,6 +383,9 @@ class one2many(_column):
     def set_memory(self, cr, obj, id, field, values, user=None, context=None):
         if not context:
             context = {}
+        if self._context:
+            context = context.copy()
+        context.update(self._context)
         if not values:
             return
         obj = obj.pool.get(self._obj)
@@ -408,19 +415,27 @@ class one2many(_column):
     def get(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
         if not context:
             context = {}
+        if self._context:
+            context = context.copy()
+        context.update(self._context)
         if not values:
             values = {}
         res = {}
         for id in ids:
             res[id] = []
-        ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id, 'in', ids)], limit=self._limit)
+        ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id, 'in', ids)], limit=self._limit, context=context)
         for r in obj.pool.get(self._obj)._read_flat(cr, user, ids2, [self._fields_id], context=context, load='_classic_write'):
             res[r[self._fields_id]].append(r['id'])
         return res
 
     def set(self, cr, obj, id, field, values, user=None, context=None):
+        result = []
         if not context:
             context = {}
+        if self._context:
+            context = context.copy()
+        context.update(self._context)
+        context['no_store_function'] = True
         if not values:
             return
         _table = obj.pool.get(self._obj)._table
@@ -428,7 +443,8 @@ class one2many(_column):
         for act in values:
             if act[0] == 0:
                 act[2][self._fields_id] = id
-                obj.create(cr, user, act[2], context=context)
+                id_new = obj.create(cr, user, act[2], context=context)
+                result += obj._store_get_values(cr, user, [id_new], act[2].keys(), context)
             elif act[0] == 1:
                 obj.write(cr, user, [act[1]], act[2], context=context)
             elif act[0] == 2:
@@ -440,13 +456,12 @@ class one2many(_column):
             elif act[0] == 5:
                 cr.execute('update '+_table+' set '+self._fields_id+'=null where '+self._fields_id+'=%s', (id,))
             elif act[0] == 6:
-                if not act[2]:
-                    ids2 = [0]
-                else:
-                    ids2 = act[2]
-                cr.execute('update '+_table+' set '+self._fields_id+'=NULL where '+self._fields_id+'=%s and id not in ('+','.join(map(str, ids2))+')', (id,))
-                if act[2]:
-                    cr.execute('update '+_table+' set '+self._fields_id+'=%s where id in ('+','.join(map(str, act[2]))+')', (id,))
+                obj.write(cr, user, act[2], {self._fields_id:id}, context=context or {})
+                ids2 = act[2] or [0]
+                cr.execute('select id from '+_table+' where '+self._fields_id+'=%s and id not in ('+','.join(map(str, ids2))+')', (id,))
+                ids3 = map(lambda x:x[0], cr.fetchall())
+                obj.write(cr, user, ids3, {self._fields_id:False}, context=context or {})
+        return result
 
     def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None, operator='like'):
         return obj.pool.get(self._obj).name_search(cr, uid, value, self._domain, offset, limit)
@@ -464,6 +479,7 @@ class one2many(_column):
 class many2many(_column):
     _classic_read = False
     _classic_write = False
+    _prefetch = False
     _type = 'many2many'
 
     def __init__(self, obj, rel, id1, id2, string='unknown', limit=None, **args):
@@ -574,6 +590,7 @@ class many2many(_column):
 class function(_column):
     _classic_read = False
     _classic_write = False
+    _prefetch = False
     _type = 'function'
     _properties = True
 
@@ -624,16 +641,29 @@ class function(_column):
         else:
             res = self._fnct(cr, obj._table, ids, name, self._arg, context)
 
+        if self._type == "many2one" :
+            # Filtering only integer/long values if passed
+            res_ids = [x for x in res.values() if x and isinstance(x, (int,long))]
+            
+            if res_ids:
+                obj_model = obj.pool.get(self._obj)
+                dict_names = dict(obj_model.name_get(cr, user, res_ids, context))
+                for r in res.keys():
+                    if res[r] and res[r] in dict_names:
+                        res[r] = (res[r], dict_names[res[r]])
+            
         if self._type == 'binary' and context.get('bin_size', False):
             # convert the data returned by the function with the size of that data...
-            res = dict(map(lambda (x, y): (x, tools.human_size(len(y))), res.items()))
+            res = dict(map(lambda (x, y): (x, tools.human_size(len(y or ''))), res.items()))
         return res
+    get_memory = get
 
     def set(self, cr, obj, id, name, value, user=None, context=None):
         if not context:
             context = {}
         if self._fnct_inv:
             self._fnct_inv(obj, cr, user, id, name, value, self._fnct_inv_arg, context)
+    set_memory = set
 
 # ---------------------------------------------------------
 # Related fields
@@ -676,8 +706,9 @@ class related(function):
                         t_data = False
                         break
                     if field_detail['type'] in ('one2many', 'many2many'):
-                        t_id=t_data.id
-                        t_data = t_data[self.arg[i]][0]
+                        if self._type != "many2one":
+                            t_id=t_data.id
+                            t_data = t_data[self.arg[i]][0]
                     else:
                         t_id=t_data['id']
                         t_data = t_data[self.arg[i]]
@@ -816,7 +847,18 @@ class property(function):
                     int(prop.value.split(',')[1])) or False
 
         obj = obj.pool.get(self._obj)
-        names = dict(obj.name_get(cr, uid, filter(None, res.values()), context))
+
+        to_check = res.values()
+        if default_val and default_val not in to_check:
+            to_check += [default_val]
+        existing_ids = obj.search(cr, uid, [('id', 'in', to_check)])
+        
+        for id, res_id in res.items():
+            if res_id not in existing_ids:
+                cr.execute('DELETE FROM ir_property WHERE value=%s', ((obj._name+','+str(res_id)),))
+                res[id] = default_val
+
+        names = dict(obj.name_get(cr, uid, existing_ids, context))
         for r in res.keys():
             if res[r] and res[r] in names:
                 res[r] = (res[r], names[res[r]])
